@@ -1,7 +1,8 @@
 # -*- coding: utf-8 -*-
+# cython: language_level=3, always_allow_keywords=True
 
-## Copyright 1999-2012 by LivingLogic AG, Bayreuth/Germany
-## Copyright 1999-2012 by Walter Dörwald
+## Copyright 1999-2014 by LivingLogic AG, Bayreuth/Germany
+## Copyright 1999-2014 by Walter Dörwald
 ##
 ## All Rights Reserved
 ##
@@ -64,9 +65,9 @@ def filter(iter, *selectors):
 
 def selector(*objs):
 	"""
-	Create a :class:`Selector` object from :var:`objs`.
+	Create a :class:`Selector` object from :obj:`objs`.
 
-	If :var:`objs` is empty (i.e. :func:`selector` is called without arguments)
+	If :obj:`objs` is empty (i.e. :func:`selector` is called without arguments)
 	``any`` is returned (which matches every node).
 
 	If more than one argument is passed (or the argument is a tuple), an
@@ -135,47 +136,47 @@ class Selector(object):
 
 	def __truediv__(self, other):
 		"""
-		Create a :class:`ChildCombinator` with :var:`self` as the left hand
-		selector and :var:`other` as the right hand selector.
+		Create a :class:`ChildCombinator` with :obj:`self` as the left hand
+		selector and :obj:`other` as the right hand selector.
 		"""
 		return ChildCombinator(self, selector(other))
 
 	def __floordiv__(self, other):
 		"""
-		Create a :class:`DescendantCombinator` with :var:`self` as the left hand
-		selector and :var:`other` as the right hand selector.
+		Create a :class:`DescendantCombinator` with :obj:`self` as the left hand
+		selector and :obj:`other` as the right hand selector.
 		"""
 		return DescendantCombinator(self, selector(other))
 
 	def __mul__(self, other):
 		"""
-		Create an :class:`AdjacentSiblingCombinator` with :var:`self` as the left
-		hand selector and :var:`other` as the right hand selector.
+		Create an :class:`AdjacentSiblingCombinator` with :obj:`self` as the left
+		hand selector and :obj:`other` as the right hand selector.
 		"""
 		return AdjacentSiblingCombinator(self, selector(other))
 
 	def __pow__(self, other):
 		"""
-		Create a :class:`GeneralSiblingCombinator` with :var:`self` as the left
-		hand selector and :var:`other` as the right hand selector.
+		Create a :class:`GeneralSiblingCombinator` with :obj:`self` as the left
+		hand selector and :obj:`other` as the right hand selector.
 		"""
 		return GeneralSiblingCombinator(self, selector(other))
 
 	def __and__(self, other):
 		"""
-		Create an :class:`AndCombinator` from :var:`self` and :var:`other`.
+		Create an :class:`AndCombinator` from :obj:`self` and :obj:`other`.
 		"""
 		return AndCombinator(self, selector(other))
 
 	def __or__(self, other):
 		"""
-		Create an :class:`OrCombinator` from :var:`self` and :var:`other`.
+		Create an :class:`OrCombinator` from :obj:`self` and :obj:`other`.
 		"""
 		return OrCombinator(self, selector(other))
 
 	def __invert__(self):
 		"""
-		Create a :class:`NotCombinator` inverting :var:`self`.
+		Create a :class:`NotCombinator` inverting :obj:`self`.
 		"""
 		return NotCombinator(self)
 
@@ -245,7 +246,7 @@ class IsInstanceSelector(Selector):
 
 	def __getitem__(self, index):
 		"""
-		Return an :class:`nthoftype` selector that uses :var:`index` as the
+		Return an :class:`nthoftype` selector that uses :obj:`index` as the
 		index and ``self.types`` as the types.
 		"""
 		return nthoftype(index, *self.types)
@@ -257,12 +258,10 @@ class IsInstanceSelector(Selector):
 			return "({})".format(" | ".join("{0.__module__}.{0.__name__}".format(type) for type in self.types))
 
 
-class hasname(Selector):
+class element(Selector):
 	"""
-	Selector that selects all nodes that have a specified Python name (which
-	only selects elements, processing instructions and entities). Also a namespace
-	name can be specified as a second argument, which will only select elements
-	from the specified namespace::
+	Selector that selects all elements that have a specified namespace name and
+	element name.::
 
 		>>> from ll.xist import xsc, parse, xfind
 		>>> from ll.xist.ns import xml, html, chars
@@ -271,7 +270,7 @@ class hasname(Selector):
 		... 	parse.Expat(ns=True),
 		... 	parse.Node(pool=xsc.Pool(xml, html, chars))
 		... )
-		>>> for node in doc.walknodes(xfind.hasname("img")):
+		>>> for node in doc.walknodes(xfind.element(html, "img")):
 		... 	print(node.string())
 		...
 		<img border="0" src="http://www.python.org/images/python-logo.gif" alt="homepage" id="logo" />
@@ -279,45 +278,46 @@ class hasname(Selector):
 		<img border="0" id="skiptocontent" alt="skip to content" src="http://www.python.org/images/trans.gif" />
 		<img alt="success story photo" class="success" src="http://www.python.org/images/success/nasa.jpg" />
 	"""
-	def __init__(self, name, xmlns=None):
-		self.name = name
+	def __init__(self, xmlns, xmlname):
 		self.xmlns = xsc.nsname(xmlns)
+		self.xmlname = xmlname
 
 	def __contains__(self, path):
 		node = path[-1]
-		if self.xmlns is not None:
-			return isinstance(node, xsc.Element) and node.__class__.__name__ == self.name and node.xmlns == self.xmlns
-		else:
-			return isinstance(node, (xsc.Element, xsc.ProcInst, xsc.Entity)) and node.__class__.__name__ == self.name
+		return isinstance(node, xsc.Element) and node.xmlns == self.xmlns and node.xmlname == self.xmlname
 
 	def __str__(self):
-		if self.xmlns is not None:
-			return "{0.__class__.__name__}({0.name!r}, {0.xmlns!r})".format(self)
-		else:
-			return "{0.__class__.__name__}({0.name!r})".format(self)
+		return "{0.__class__.__name__}({0.name!r}, {0.xmlns!r})".format(self)
 
 
-class hasname_xml(Selector):
+class procinst(Selector):
 	"""
-	:class:`hasname_xml` works similar to :class:`hasname` except that the
-	specified name is treated as the XML name, not the Python name.
+	Selector that selects all processing instructions that have a specified name.
 	"""
-	def __init__(self, name, xmlns=None):
-		self.name = name
-		self.xmlns = xsc.nsname(xmlns)
+	def __init__(self, xmlname):
+		self.xmlname = xmlname
 
 	def __contains__(self, path):
 		node = path[-1]
-		if self.xmlns is not None:
-			return isinstance(node, xsc.Element) and node.xmlname == self.name and node.xmlns == self.xmlns
-		else:
-			return isinstance(node, (xsc.Element, xsc.ProcInst, xsc.Entity)) and node.xmlname == self.name
+		return isinstance(node, xsc.ProcInst) and node.xmlname == self.xmlname
 
 	def __str__(self):
-		if self.xmlns is not None:
-			return "{0.__class__.__name__}({0.name!r}, {0.xmlns!r})".format(self)
-		else:
-			return "{0.__class__.__name__}({0.name!r})".format(self)
+		return "{0.__class__.__name__}({0.name!r})".format(self)
+
+
+class entity(Selector):
+	"""
+	Selector that selects all entities that have a specified name.
+	"""
+	def __init__(self, xmlname):
+		self.xmlname = xmlname
+
+	def __contains__(self, path):
+		node = path[-1]
+		return isinstance(node, xsc.Entity) and node.xmlname == self.xmlname
+
+	def __str__(self):
+		return "{0.__class__.__name__}({0.name!r})".format(self)
 
 
 class IsSelector(Selector):
@@ -420,7 +420,7 @@ class OnlyChildSelector(Selector):
 		if len(path) >= 2:
 			parent = path[-2]
 			if isinstance(parent, (xsc.Frag, xsc.Element)):
-				return len(parent)==1 and parent[0] is path[-1]
+				return len(parent) == 1 and parent[0] is path[-1]
 		return False
 
 	def __str__(self):
@@ -473,9 +473,9 @@ onlyoftype = OnlyOfTypeSelector()
 
 class hasattr(Selector):
 	"""
-	Selector that selects all element nodes that have an attribute with one
-	of the specified Python names. For selecting nodes with global attributes
-	the attribute class can be passed::
+	Selector that selects all element nodes that have an attribute with one of
+	the specified names. (Names can be strings, (attribute name, namespace name)
+	tuples or attribute classes or instances)::
 
 		>>> from ll.xist import xsc, parse, xfind
 		>>> from ll.xist.ns import xml, html, chars
@@ -497,7 +497,7 @@ class hasattr(Selector):
 		node = path[-1]
 		if isinstance(node, xsc.Element):
 			for attrname in self.attrnames:
-				if node.Attrs.isallowed(attrname) and node.attrs.has(attrname):
+				if attrname in node.attrs:
 					return True
 		return False
 
@@ -505,33 +505,13 @@ class hasattr(Selector):
 		return "{}({})".format(self.__class__.__name__, ", ".join(repr(attrname) for attrname in self.attrnames))
 
 
-class hasattr_xml(Selector):
-	"""
-	:class:`hasattr_xml` works similar to :class:`hasattr` except that the
-	specified names are treated as XML names instead of Python names.
-	"""
-
-	def __init__(self, *attrnames):
-		self.attrnames = attrnames
-
-	def __contains__(self, path):
-		node = path[-1]
-		if isinstance(node, xsc.Element):
-			for attrname in self.attrnames:
-				if node.Attrs.isallowed_xml(attrname) and node.attrs.has_xml(attrname):
-					return True
-		return False
-
-	def __str__(self):
-		return "{0.__class__.__name__}({1})".format(self, ", ".join(repr(attrname) for attrname in self.attrnames))
-
-
 class attrhasvalue(Selector):
 	"""
-	Selector that selects all element nodes where an attribute with the
-	specified Python name has one of the specified values. For global attributes
-	the attribute class can be passed. Note that "fancy" attributes (i.e. those
-	containing non-text) will not be considered::
+	Selector that selects all element nodes where an attribute with the specified
+	name has one of the specified values. (Names can be strings,
+	(attribute name, namespace name) tuples or attribute classes or instances).
+	Note that "fancy" attributes (i.e. those containing non-text) will not be
+	considered::
 
 		>>> from ll.xist import xsc, parse, xfind
 		>>> from ll.xist.ns import xml, html, chars
@@ -556,7 +536,7 @@ class attrhasvalue(Selector):
 
 	def __contains__(self, path):
 		node = path[-1]
-		if isinstance(node, xsc.Element) and node.Attrs.isallowed(self.attrname):
+		if isinstance(node, xsc.Element):
 			attr = node.attrs.get(self.attrname)
 			if not attr.isfancy(): # if there are PIs, say no
 				return str(attr) in self.attrvalues
@@ -566,36 +546,13 @@ class attrhasvalue(Selector):
 		return "{0.__class__.__name__}({0.attrname!r}, {1})".format(self, repr(self.attrvalues)[1:-1])
 
 
-class attrhasvalue_xml(Selector):
-	"""
-	:class:`attrhasvalue_xml` works similar to :class:`attrhasvalue` except that
-	the specified name is treated as an XML name instead of a Python name.
-	"""
-
-	def __init__(self, attrname, *attrvalues):
-		self.attrname = attrname
-		if not attrvalues:
-			raise ValueError("need at least one attribute value")
-		self.attrvalues = attrvalues
-
-	def __contains__(self, path):
-		node = path[-1]
-		if isinstance(node, xsc.Element) and node.Attrs.isallowed_xml(self.attrname):
-			attr = node.attrs.get_xml(self.attrname)
-			if not attr.isfancy(): # if there are PIs, say no
-				return str(attr) in self.attrvalues
-		return False
-
-	def __str__(self):
-		return "{0.__class__.__name__}({0.attrname!1}, {0.attrvalue!r})".format(self)
-
-
 class attrcontains(Selector):
 	"""
-	Selector that selects all element nodes where an attribute with the
-	specified Python name contains one of the specified substrings in its value.
-	For global attributes the attribute class can be passed. Note that "fancy"
-	attributes (i.e. those containing non-text) will not be considered::
+	Selector that selects all element nodes where an attribute with the specified
+	name contains one of the specified substrings in its value. (Names can be
+	strings, (attribute name, namespace name) tuples or attribute classes or
+	instances). Note that "fancy" attributes (i.e. those containing non-text)
+	will not be considered::
 
 		>>> from ll.xist import xsc, parse, xfind
 		>>> from ll.xist.ns import xml, html, chars
@@ -623,7 +580,7 @@ class attrcontains(Selector):
 
 	def __contains__(self, path):
 		node = path[-1]
-		if isinstance(node, xsc.Element) and node.Attrs.isallowed(self.attrname):
+		if isinstance(node, xsc.Element):
 			attr = node.attrs.get(self.attrname)
 			if not attr.isfancy(): # if there are PIs, say no
 				return builtins.any(attrvalue in str(attr) for attrvalue in self.attrvalues)
@@ -633,36 +590,13 @@ class attrcontains(Selector):
 		return "{0.__class__.__name__}({0.attrname!1}, {0.attrvalue!r})".format(self)
 
 
-class attrcontains_xml(Selector):
-	"""
-	:class:`attrcontains_xml` works similar to :class:`attrcontains` except that
-	the specified name is treated as an XML name instead of a Python name.
-	"""
-
-	def __init__(self, attrname, *attrvalues):
-		self.attrname = attrname
-		if not attrvalues:
-			raise ValueError("need at least one attribute value")
-		self.attrvalues = attrvalues
-
-	def __contains__(self, path):
-		node = path[-1]
-		if isinstance(node, xsc.Element) and node.Attrs.isallowed_xml(self.attrname):
-			attr = node.attrs.get_xml(self.attrname)
-			if not attr.isfancy(): # if there are PIs, say no
-				return builtins.any(attrvalue in str(attr) for attrvalue in self.attrvalues)
-		return False
-
-	def __str__(self):
-		return "{0.__class__.__name__}({0.attrname!1}, {0.attrvalue!r})".format(self)
-
-
 class attrstartswith(Selector):
 	"""
-	Selector that selects all element nodes where an attribute with the
-	specified Python name starts with any of the specified strings. For global
-	attributes the attribute class can be passed. Note that "fancy" attributes
-	(i.e. those containing non-text) will not be considered::
+	Selector that selects all element nodes where an attribute with the specified
+	name starts with any of the specified strings. (Names can be strings,
+	(attribute name, namespace name) tuples or attribute classes or instances).
+	Note that "fancy" attributes (i.e. those containing non-text) will not be
+	considered::
 
 		>>> from ll.xist import xsc, parse, xfind
 		>>> from ll.xist.ns import xml, html, chars
@@ -686,7 +620,7 @@ class attrstartswith(Selector):
 
 	def __contains__(self, path):
 		node = path[-1]
-		if isinstance(node, xsc.Element) and node.Attrs.isallowed(self.attrname):
+		if isinstance(node, xsc.Element):
 			attr = node.attrs.get(self.attrname)
 			if not attr.isfancy(): # if there are PIs, say no
 				return builtins.any(str(attr).startswith(attrvalue) for attrvalue in self.attrvalues)
@@ -696,36 +630,13 @@ class attrstartswith(Selector):
 		return "{0.__class__.__name__}({0.attrname!1}, {0.attrvalue!r})".format(self)
 
 
-class attrstartswith_xml(Selector):
-	"""
-	:class:`attrstartswith_xml` works similar to :class:`attrstartswith` except
-	that the specified name is treated as an XML name instead of a Python name.
-	"""
-
-	def __init__(self, attrname, *attrvalues):
-		self.attrname = attrname
-		if not attrvalues:
-			raise ValueError("need at least one attribute value")
-		self.attrvalues = attrvalues
-
-	def __contains__(self, path):
-		node = path[-1]
-		if isinstance(node, xsc.Element) and node.Attrs.isallowed_xml(self.attrname):
-			attr = node.attrs.get_xml(self.attrname)
-			if not attr.isfancy(): # if there are PIs, say no
-				return builtins.any(str(attr).startswith(attrvalue) for attrvalue in self.attrvalues)
-		return False
-
-	def __str__(self):
-		return "{0.__class__.__name__}({0.attrname!1}, {0.attrvalue!r})".format(self)
-
-
 class attrendswith(Selector):
 	"""
-	Selector that selects all element nodes where an attribute with the
-	specified Python name ends with one of the specified strings. For global
-	attributes the attribute class can be passed. Note that "fancy" attributes
-	(i.e. those containing non-text) will not be considered::
+	Selector that selects all element nodes where an attribute with the specified
+	name ends with one of the specified strings. (Names can be strings,
+	(attribute name, namespace name) tuples or attribute classes or instances).
+	Note that "fancy" attributes (i.e. those containing non-text) will not be
+	considered::
 
 		>>> from ll.xist import xsc, parse, xfind
 		>>> from ll.xist.ns import xml, html, chars
@@ -752,7 +663,7 @@ class attrendswith(Selector):
 
 	def __contains__(self, path):
 		node = path[-1]
-		if isinstance(node, xsc.Element) and node.Attrs.isallowed(self.attrname):
+		if isinstance(node, xsc.Element):
 			attr = node.attrs.get(self.attrname)
 			if not attr.isfancy(): # if there are PIs, say no
 				return builtins.any(str(attr).endswith(attrvalue) for attrvalue in self.attrvalues)
@@ -762,30 +673,6 @@ class attrendswith(Selector):
 		return "{0.__class__.__name__}({0.attrname!1}, {0.attrvalue!r})".format(self)
 
 
-class attrendswith_xml(Selector):
-	"""
-	:class:`attrendswith_xml` works similar to :class:`attrendswith` except that
-	the specified name is treated as an XML name instead of a Python name.
-	"""
-
-	def __init__(self, attrname, *attrvalues):
-		self.attrname = attrname
-		if not attrvalues:
-			raise ValueError("need at least one attribute value")
-		self.attrvalues = attrvalues
-
-	def __contains__(self, path):
-		node = path[-1]
-		if isinstance(node, xsc.Element) and node.Attrs.isallowed_xml(self.attrname):
-			attr = node.attrs.get_xml(self.attrname)
-			if not attr.isfancy(): # if there are PIs, say no
-				return builtins.any(str(attr).endswith(attrvalue) for attrvalue in self.attrvalues)
-		return False
-
-	def __str__(self):
-		return "{0.__class__.__name__}({0.attrname!1}, {0.attrvalue!r})".format(self)
-
-
 class hasid(Selector):
 	"""
 	Selector that selects all element nodes where the ``id`` attribute has one
@@ -811,8 +698,8 @@ class hasid(Selector):
 
 	def __contains__(self, path):
 		node = path[-1]
-		if isinstance(node, xsc.Element) and node.Attrs.isallowed_xml("id"):
-			attr = node.attrs.get_xml("id")
+		if isinstance(node, xsc.Element):
+			attr = node.attrs.get("id")
 			if not attr.isfancy():
 				return str(attr) in self.ids
 		return False
@@ -850,8 +737,8 @@ class hasclass(Selector):
 
 	def __contains__(self, path):
 		node = path[-1]
-		if isinstance(node, xsc.Element) and node.Attrs.isallowed_xml("class"):
-			attr = node.attrs.get_xml("class")
+		if isinstance(node, xsc.Element):
+			attr = node.attrs.get("class")
 			if not attr.isfancy():
 				return builtins.any(classname in str(attr).split() for classname in self.classnames)
 		return False
